#!/usr/bin/env python

#############################################################
#                                                           #
# freeleech-dl - what.cd freeleech downloader for ruTorrent #
# written by Hunter Hammond (huntrar@gmail.com)             #
#                                                           #
#############################################################


import argparse as argp
import getpass
import random
import time

from selenium import webdriver
from selenium.webdriver.common.keys import Keys

from . import __version__


USER_AGENTS = ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) '
               'Gecko/20100101 Firefox/11.0',
               'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:22.0) '
               'Gecko/20100 101 Firefox/22.0',
               'Mozilla/5.0 (Windows NT 6.1; rv:11.0) '
               'Gecko/20100101 Firefox/11.0',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) '
               'AppleWebKit/536.5 (KHTML, like Gecko) '
               'Chrome/19.0.1084.46 Safari/536.5',
               'Mozilla/5.0 (Windows; Windows NT 6.1) '
               'AppleWebKit/536.5 (KHTML, like Gecko) '
               'Chrome/19.0.1084.46 Safari/536.5')


def get_parser():
    parser = argp.ArgumentParser(description='what.cd freeleech downloader for \
                                             ruTorrent')
    parser.add_argument('-rt', '--rutorrent', type=str,
                        help='ruTorrent web portal url')
    parser.add_argument('-v', '--version', help='display current version',
                        action='store_true')
    return parser


def get_driver():
    profile = webdriver.FirefoxProfile()
    profile.set_preference('general.useragent.override\
                           ', random.choice(USER_AGENTS))
    return webdriver.Firefox(profile)


def what_login():
    user = raw_input('what.cd username: ')
    if not user:
        user = getpass.getuser()

    password = getpass.getpass()

    if user and password:
        url = 'https://what.cd/login.php' 

        driver = get_driver()
        driver.get(url)
        element = driver.find_element_by_xpath('//input[@id="username"]')
        element.send_keys(user)
        element = driver.find_element_by_xpath('//input[@id="password"]')
        element.send_keys(password)
        element.send_keys(Keys.RETURN)

        ''' Give time to login '''
        time.sleep(5)
        return driver
    return False


def get_torrent_links():
    driver = what_login()
    if driver:
        url = 'https://what.cd/torrents.php?page=1&freetorrent=1'
        driver.get(url)

        ''' Get DL links '''
        elements = driver.find_elements_by_link_text('DL')
        if elements:
            return [link.get_attribute('href') for link in elements]
    return False


def main():
    parser = get_parser()
    args = vars(parser.parse_args())

    if args['version']:
        print(__version__)
        return

    if not args['rutorrent']:
        parser.print_help()
        return
    else:
        rt_url = args['rutorrent']

        ''' Remove scheme if possible '''
        if rt_url.startswith('http://'):
            rt_url = rt_url.replace('http://', '')
        elif rt_url.startswith('https://'):
            rt_url = rt_url.replace('https://', '')

    user = raw_input('ruTorrent username: ')
    if not user:
        user = getpass.getuser()

    password = getpass.getpass()

    torrents = get_torrent_links()
    if torrents and user and password:
        rt_url = 'http://{0}:{1}@{2}'.format(user, password, rt_url)

        driver = get_driver()

        ''' Make WebDriver poll the DOM if elements not immediately found '''
        driver.implicitly_wait(5) # seconds 

        driver.get(rt_url)

        ''' Click the Add Torrent button in ruTorrent web client '''
        element = driver.find_element_by_xpath('//a[@id="mnu_add"]')
        element.click()

        for url in torrents:
            ''' Enter Torrent URL '''
            print('Added {0} to ruTorrent'.format(url))
            element = driver.find_element_by_xpath('//form[@id="addtorrenturl"]\
                                                   /input[@id="url"]')
            element.send_keys(url)
            element.send_keys(Keys.RETURN)

            ''' Give time to load the added torrent '''
            time.sleep(5)



if __name__ == '__main__':
    main()
